# Generated by roxygen2: do not edit by hand

export(Amatrix)
export(AmatrixPolyCross)
export(Gmatrix)
export(Hmatrix)
export(Mcheck)
export(asciitonumber)
export(datatreat)
export(expandAmatrix)
export(filterpedigree)
export(formatmatrix)
export(missingdata)
importFrom(Rcpp,evalCpp)
importFrom(matrixStats,colVars)
importFrom(stats,approx)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,write.table)
useDynLib(AGHmatrix, .registration = TRUE)
