#' Pedigree Data
#'
#' Data from pedigree example proposed by Mrode 2005
#'
#' @docType data
#'
#' @usage data(ped.mrode)
#'
#' @format table
#'
#' @keywords datasets
#'
#'
#' @references R. A. Mrode, R. Thompson. Linear Models for the Prediction of Animal Breeding Values. CABI, 2005.
#'
#' @examples data(ped.mrode)
#'
#' @name ped.mrode
NULL
