#' Molecular data for autopolyploid examples
#'
#' Dataset extract from  supplementary material from Endelman et al. (2018). 
#' SNP marker matrix from Pine tree coded as 0,1,2,3,4 and missing value as -9.
#'
#' @docType data
#'
#' @usage data(snp.sol)
#'
#' @format data.frame
#'
#' @keywords datasets
#'
#' @references Endelman, JB, et al., 2018 Genetic variance partitioning and genome-wide prediction with allele dosage information in autotetraploid potato. Genetics, 209(1) pp. 77-87.
#' 
#' @examples data(snp.sol)
#'
#' @name snp.sol
NULL
