#' Molecular data for diploid examples
#'
#' Dataset extract from  supplementary material from Resende et al. (2012). 
#' SNP marker matrix from Pine tree coded as 0,1, and 2, and missing value as -9.
#'
#' @docType data
#'
#' @usage data(snp.pine)
#'
#' @format matrix
#'
#' @keywords datasets
#'
#' @references Resende, MF, et al., 2012 Accuracy of genomic selection methods in a standard data set of loblolly pine (Pinus taeda l.). Genetics 190: 1503–1510.
#'
#' @examples data(snp.pine)
#'
#' @name snp.pine
NULL
